from .. import Provider as BankProvider

# https://www.bcb.gov.br/estabilidadefinanceira/iban
# The Central Bank published Circular No. 3,625 on February 14, 2013, which
# regulates the use of the International Bank Account Number (IBAN) as a
# standard identifier for international transfers of funds to bank accounts
# held in Brazil.


class Provider(BankProvider):
    """Implement bank provider for ``pt_BR`` locale."""

    bban_format: str = "#######################??"  # BRXX + 23 digits + 2 alphanumeric
    country_code: str = "BR"

    banks = (
        "AGORACRED S/A Sociedade de Crédito, financiamento e Investimento",
        "AL5 S/A Sociedade de Crédito, financiamento e Investimento",
        "ATIVA INVESTIMENTOS S/A Corretora de Títulos, Câmbio e Valores",
        "BECKER Financeira S/A - Crédito, financiamento e Investimento",
        "BPY Corretora de Câmbio, Títulos e Valores Mobiliários S/A",
        "CARUANA S/A - Sociedade de Crédito, financiamento e Investimento",
        "CM CAPITAL MARKETS Corretora de Câmbio, Títulos e Valores Mobiliários Ltda",
        "COBUCCIO S/A - Sociedade de Crédito, financiamento e Investimentos",
        "Central SICREDI BRASIL CENTRAL",
        "Central SICREDI CENTRO NORTE",
        "Central SICREDI NORDESTE",
        "Central SICREDI SUL/SUDESTE",
        "Central SICREDI PR/SP/RJ",
        "CORA Sociedade de Crédito, financiamento e Investimento S/A",
        "DM Sociedade de Crédito, financiamento e Investimento S/A",
        "DUFRIO Financeira, Crédito, financiamento e Investimentos S/A",
        "FINAMAX S/A - Crédito, financiamento e Investimento",
        "GAZINCRED S/A Sociedade de Crédito, financiamento e Investimento",
        "HARMOS S/A - Sociedade de Crédito, financiamento e Investimento",
        "HS Financeira S/A Crédito, financiamento e Investimentos",
        "J17 - Sociedade de Crédito, financiamento e Investimento S/A",
        "JPMorgan Chase Bank, National Association",
        "KANASTRA Financeira S/A, Crédito, financiamento e Investimento",
        "Lecca Crédito, financiamento e Investimento S/A",
        "MAGALUPAY - Sociedade de Crédito, financiamento e Investimento S/A",
        "MERCADO CRÉDITO Sociedade de Crédito, financiamento e Investimento S/A",
        "MERCANTIL Financeira S/A - Sociedade de Crédito, financiamento e Investimento.",
        "MIDWAY S/A - Sociedade de Crédito, financiamento e Investimento",
        "MIRAE ASSET (BRASIL) Corretora de Câmbio, Títulos e Valores Mobiliários Ltda",
        "MULTIPLIKE Financeira S/A Sociedade de Crédito, financiamento e Investimento",
        "NEON Financeira - Sociedade de Crédito, financiamento e Investimento S/A",
        "NU Financeira S/A - Sociedade de Crédito, financiamento e Investimento",
        "PAN Financeira S/A - Sociedade de Crédito, financiamento e Investimento",
        "PARATI - Crédito, financiamento e Investimento S/A",
        "PEFISA S/A - Crédito, financiamento e Investimento",
        "PLANTAE S/A - Crédito, financiamento e Investimento",
        "PORTOSEG S/A - Crédito, financiamento e Investimento",
        "QISTA S/A - Crédito, financiamento e Investimento",
        "REALIZE Sociedade de Crédito, financiamento e Investimento S/A",
        "RPW S/A Sociedade de Crédito, financiamento e Investimento",
        "SANTINVEST S/A - Crédito, financiamento e Investimentos",
        "Sociedade de Crédito, financiamento e Investimento EFÍ S/A",
        "SOCINAL S/A - Crédito, financiamento e Investimento",
        "STONE Sociedade de Crédito, financiamento e Investimento S/A",
        "UBS BB Corretora de Câmbio, Títulos e Valores Mobiliários S/A",
        "URBANO S/A - Sociedade de Crédito, financiamento e Investimento",
        "VALOR S/A Sociedade de Crédito, financiamento e Investimento",
        "Via Certa Financiadora S/A - Crédito, financiamento e Investimentos",
        "XP INVESTIMENTOS Corretora de Câmbio, Títulos e Valores S/A",
        "ZEMA Crédito, financiamento e Investimento S/A",
        "321 Sociedade de Crédito Direto S/A",
        "99PAY Instituição de Pagamento S/A",
        "A55 Sociedade de Crédito Direto S/A",
        "ACCREDITO - Sociedade de Crédito Direto S/A",
        "ACESSO Soluções DE Pagamento S/A - Instituição de Pagamento",
        "ACTUAL Distribuidora de Títulos e Valores Mobiliários S/A",
        "AGK Corretora de Câmbio S/A",
        "ALL IN CRED Sociedade de Crédito Direto S/A",
        "APUSDIGITAL Instituição de Pagamento Ltda",
        "ARTTA Sociedade de Crédito Direto S/A",
        "ASA Sociedade de Crédito, Financiamento e Investimento S/A",
        "ASAAS GESTÃO Financeira Instituição de Pagamento S/A",
        "ATF Sociedade de Crédito Direto S/A",
        "ATICCA - Sociedade de Crédito Direto S/A",
        "ATTRUS Instituição de Pagamento S/A",
        "ATUAL Sociedade Corretora de Câmbio Ltda",
        "AVANCARD PROVER Instituição de Pagamento Ltda",
        "AVENUE SECURITIES Banco de Investimentos/A",
        "AWX BRASIL  Instituição de Pagamento Ltda",
        "AZIMUT BRASIL Corretora de Títulos e Valores Mobiliários Ltda",
        "AZUMI Corretora de Títulos e Valores Mobiliários Ltda",
        "Banco ABN AMRO Clearing S/A",
        "Banco AFINZ S/A - Banco Múltiplo",
        "Banco BARI De Investimentos e Financiamentos S/A",
        "Banco BESA S/A",
        "Banco BMG Consignado S/A",
        "Banco BMG Soluções FinanceiraS S/A",
        "Banco BRASILEIRO de Crédito S/A",
        "Banco BV S/A",
        "Banco C6 Consignado S/A",
        "Banco CEDULA S/A",
        "Banco CLASSICO S/A",
        "Banco COOPERATIVO SICOOB S/A - Banco SICOOB",
        "Banco COOPERATIVO SICREDI S/A",
        "Banco CRÉDIT AGRICOLE BRASIL S/A",
        "Banco DA AMAZONIA S/A",
        "Banco DIGIMAIS S/A",
        "Banco GENIAL S/A",
        "Banco GM S/A",
        "Banco HSBC S/A",
        "Banco INDUSCRED de Investimentos/A",
        "Banco J.P. MORGAN S/A",
        "Banco KEB HANA DO BRASIL S/A",
        "Banco MERCEDES-BENZ DO BRASIL S/A",
        "Banco MORGAN STANLEY S/A",
        "Banco NACIONAL DE DESENVOLVIMENTO ECONOMICO e SOCIAL",
        "Banco NACIONAL S/A",
        "Banco RANDON S/A",
        "Banco RIBEIRAO PRETO S/A",
        "Banco RNX S/A",
        "Banco RODOBENS S/A",
        "Banco SANTANDER (BRASIL) S/A",
        "Banco SENFF S/A",
        "Banco SOCIETE GENERALE BRASIL S/A",
        "Banco SOFISA S/A",
        "Banco STONEX S/A",
        "Banco TOPÁZIO S/A",
        "Banco TRATON BRASIL S/A",
        "Banco TRAVELEX S/A",
        "Banco TRIANGULO S/A",
        "Banco UBS (BRASIL) S/A",
        "BANESTES S/A Banco DO ESTADO DO ESPIRITO SANTO",
        "BANK OF CHINA (BRASIL) Banco Múltiplo S/A",
        "BANSUR JM Sociedade de Crédito Direto S/A",
        "BANVOX Corretora de Títulos e Valores Mobiliários Ltda",
        "BARI Companhia Hipotecária",
        "BARU Corretora de Títulos e Valores Mobiliários Ltda",
        "BBVA BRASIL Banco de Investimentos/A",
        "BEES Instituição de Pagamento Ltda",
        "BEETELLER Instituição de Pagamento Ltda",
        "BFC Sociedade de Crédito Direto S/A",
        "BGC LIQUIDEZ Corretora de Títulos e Valores Mobiliários Ltda",
        "BIT Sociedade de Crédito Direto S/A",
        "BIZ Instituição de Pagamento S/A",
        "BK Instituição de Pagamento S/A",
        "BMP Sociedade de Crédito ao Microepreendedor e a Empresa de Pequeno Porte Ltda",
        "BMP Sociedade de Crédito Direto S/A",
        "BMS Sociedade de Crédito Direto S/A",
        "BNK DIGITAL Sociedade de Crédito Direto S/A",
        "BNY Mellon Banco S/A",
        "BONUSPAGO Sociedade de Crédito Direto S/A",
        "BR Partners Banco de Investimentos/A",
        "BR-CAPITAL Distribuidora de Títulos e Valores Mobiliários S/A",
        "BRASIL CASH Instituição de Pagamento S/A",
        "BRAZA BANK S/A Banco de Câmbio",
        "BRB - Banco DE BRASILIA S/A",
        "BRCONDOS Sociedade de Crédito Direto S/A",
        "BRL Trust Distribuidora de Títulos e Valores Mobiliários S/A",
        "BSN PagamentoS Instituição de Pagamento Ltda",
        "BT Corretora de Câmbio Ltda",
        "Banco ABC Brasil S/A",
        "Banco Agibank S/A",
        "Banco Alfa S/A",
        "Banco AndBank (Brasil) S/A",
        "Banco Arbi S/A",
        "Banco B3 S/A",
        "Banco BMG S/A",
        "Banco BNP Paribas Brasil S/A",
        "Banco BS2 S/A",
        "Banco BTG Pactual S/A",
        "Banco Bandepe S/A",
        "Banco Bocom BBM S/A",
        "Banco Bradescard S/A",
        "Banco Bradesco BBI S/A",
        "Banco Bradesco BERJ S/A",
        "Banco Bradesco Financiamentos S/A",
        "Banco Bradesco S/A",
        "Banco C6 S/A",
        "Banco CSF S/A",
        "Banco Caixa Geral - Brasil S/A",
        "Banco Cargill S/A",
        "Banco Central do Brasil",
        "Banco Citibank S/A",
        "Banco Crefisa S/A",
        "Banco Daycoval S/A",
        "Banco Digio S/A",
        "Banco Fator S/A",
        "Banco Fibra S/A",
        "Banco Finaxis S/A",
        "Banco Guanabara S/A",
        "Banco Inbursa S/A",
        "Banco Industrial do Brasil S/A",
        "Banco Inter S/A",
        "Banco Investcred Unibanco S/A",
        "Banco ItauBank S/A",
        "Banco J. Safra S/A",
        "Banco John Deere S/A",
        "Banco KDB do Brasil S/A",
        "Banco Luso Brasileiro S/A",
        "Banco MUFG Brasil S/A",
        "Banco Mercantil do Brasil S/A",
        "Banco Mizuho do Brasil S/A",
        "Banco Original S/A",
        "Banco Pan S/A",
        "Banco Paulista S/A",
        "Banco Pine S/A",
        "Banco Rabobank International Brasil S/A",
        "Banco Rendimento S/A",
        "Banco Safra S/A",
        "Banco Semear S/A",
        "Banco Sistema S/A",
        "Banco Sumitomo Mitsui Brasileiro S/A",
        "Banco Toyota do Brasil S/A",
        "Banco Tricury S/A",
        "Banco VR S/A",
        "Banco Volkswagen S/A",
        "Banco Votorantim S/A",
        "Banco Western Union do Brasil S/A",
        "Banco Woori Bank do Brasil S/A",
        "Banco XP S/A",
        "Banco Yamaha Motor do Brasil S/A",
        "Banco da China Brasil S/A",
        "Banco de La Provincia de Buenos Aires",
        "Banco de la Nacion Argentina",
        "Banco do Brasil S/A",
        "Banco do Estado de Sergipe S/A",
        "Banco do Estado do Pará S/A",
        "Banco do Estado do Rio Grande do Sul S/A",
        "Banco do Nordeste do Brasil S/A",
        "BancoSeguro S/A",
        "Bank of America Merrill Lynch Banco Múltiplo S/A",
        "Broker Brasil Corretora de Câmbio Ltda",
        "CACTVS Instituição de Pagamento S/A",
        "CAIXA ECONOMICA FEDERAL",
        "CAPITAL CONSIG Sociedade de Crédito Direto S/A",
        "CAROL Distribuidora de Títulos e Valores Mobiliários Ltda",
        "CARTOS Sociedade de Crédito Direto S/A",
        "CASAS BAHIA PAY Instituição de Pagamento Ltda",
        "CDC Sociedade de Crédito Direto S/A",
        "CELCOIN Instituição de Pagamento S/A",
        "CELCOIN Sociedade de Crédito Direto S/A",
        "CERC S/A",
        "CIELO S/A - Instituição de Pagamento",
        "CIP S/A",
        "CLOUDWALK Instituição de Pagamento e Serviços Ltda",
        "COLUNA S/A Distribuidora de Títulos e Valores Mobiliários",
        "CONFIDENCE Sociedade Corretora de Câmbio S/A",
        "CONTA PRONTA Instituição de Pagamento Ltda",
        "CONTAAZUL Instituição de Pagamento Ltda",
        "Cooperativa CENTRAL de Crédito UNICRED DO BRASIL - UNICRED DO BRASIL",
        "Cooperativa de Crédito Rural DE IBIAM - SULCREDI/IBIAM",
        "Cooperativa de Crédito Rural SEARA - CREDISEARA",
        "Cooperativa de Crédito SULCREDI AMPLEA",
        "COOPERFORTE Cooperativa de Crédito e Investimentos Ltda",
        "CREDI&GENTE - Cooperativa de Crédito e Investimentos",
        "CREDICOAMO CREDITO Rural Cooperativa",
        "CREDIFIT Sociedade de Crédito Direto S/A",
        "CREDISAN Cooperativa de Crédito",
        "CREDSYSTEM Sociedade de Crédito Direto S/A",
        "CREFAZ Sociedade de Crédito ao Microepreendedor e a Empresa de Pequeno Porte S/A",
        "Casa do Crédito S/A Sociedade de Crédito ao Microempreendedor",
        "Citibank N.A.",
        "Codepe Corretora de Valores e Câmbio S/A",
        "Cooperativa Central de Crédito - Ailos",
        "Cooperativa de Crédito Rural Coopavel",
        "Cooperativa de Crédito Rural de Abelardo Luz - Sulcredi/Crediluz",
        "Credisis - Central de Cooperativas de Crédito Ltda",
        "Creditas Sociedade de Crédito Direto S/A",
        "DELFINANCE Sociedade de Crédito Direto S/A",
        "DELTA GLOBAL Sociedade de Crédito Direto S/A",
        "DEUTSCHE BANK S/A - Banco ALEMAO",
        "DGBK CREDIT S/A - Sociedade de Crédito Direto.",
        "DM Sociedade de Crédito Direto S/A",
        "DOCK Instituição de Pagamento S/A",
        "EAGLE Instituição de Pagamento Ltda",
        "EBANX Instituição de PagamentoS Ltda",
        "EBURY Banco de Câmbio S/A",
        "EFEX Instituição de Pagamento S/A",
        "EFX Corretora de Câmbio Ltda",
        "EFÍ S/A - Instituição de Pagamento",
        "EMCASH Serviços Financeiros Sociedade de Empréstimo entre Pessoas S/A",
        "EWALLY Instituição de Pagamento S/A",
        "EXIM Corretora de Câmbio Ltda",
        "F.D'GOLD - Corretora de Títulos e Valores Mobiliários Ltda",
        "FAIR Sociedade Corretora de Câmbio S/A",
        "FESTOR Instituição de Pagamento Ltda",
        "FFCRED Sociedade de Crédito Direto S/A.",
        "FIDD Corretora de Títulos e Valores Mobiliários Ltda",
        "FIDEM Sociedade de Crédito Direto S/A",
        "FIDÚCIA Sociedade de Crédito ao Microempreendedor e à Empresa de Pequeno Porte Ltda.",
        "FINVEST Corretora de Títulos e Valores Mobiliários Ltda",
        "FITS Instituição de Pagamento S/A",
        "FLAGSHIP Instituição de Pagamento Ltda",
        "FOURTRADE Corretora de Câmbio Ltda",
        "FREEX Sociedade Corretora de Câmbio S/A",
        "FUNDO GARANTIDOR de CréditoS - FGC",
        "FUTURO Sociedade de Crédito Direto S/A",
        "Facta Financeira S/A - Crédito, Financiamento e Investimento",
        "G5 Sociedade de Crédito Direto S/A",
        "GALAPAGOS CAPITAL Distribuidora de Títulos e Valores Mobiliários S/A",
        "GETNET ADQUIRÊNCIA e SERVIÇOS PARA MEIOS DE Pagamento S/A Instituição de Pagamento",
        "GIRO - Sociedade de Crédito Direto S/A",
        "GLOBAL FINANÇAS Sociedade de Crédito ao Microempreendedor e à Empresa de Pequeno Porte Ltda",
        "GOLDMAN SACHS DO BRASIL Banco MULTIPLO S/A",
        "GUITTA Corretora de Câmbio Ltda",
        "Genial Investimentos Corretora de Valores Mobiliários S/A",
        "Get Money Corretora de Câmbio S/A",
        "HBI Sociedade de Crédito Direto S/A",
        "HEDGE INVESTMENTS Corretora de Títulos e Valores Mobiliários Ltda",
        "HEMERA Corretora de Títulos e Valores Mobiliários Ltda",
        "HINOVA PAY Instituição de Pagamento S/A",
        "HR DIGITAL - Sociedade de Crédito Direto S/A",
        "HSCM - Sociedade de Crédito ao Microempreendedor e à Empresa de Pequeno Porte Ltda",
        "Haitong Banco de Investimentodo Brasil S/A",
        "ICAP do Brasil Corretora de Títulos e Valores Mobiliários Ltda",
        "ICBC do Brasil Banco Múltiplo S/A",
        "ID Corretora de Títulos e Valores Mobiliários S/A",
        "IDEA MAKER Instituição de Pagamento Ltda",
        "IDEAL Corretora de Títulos e Valores Mobiliários S/A",
        "IFOOD PAGO Instituição de Pagamento S/A",
        "INCO Sociedade de Empréstimo entre Pessoas S/A",
        "INDEPENDÊNCIA Cooperativa de Crédito e Investimento - INDEPENDÊNCIA Cooperativa de Crédito",
        "Instituição de PagamentoS MAPS Ltda",
        "INTEGRAÇÃO de Crédito e Cobrança Sociedade de Crédito Direto S/A",
        "INTERCAM Corretora de Câmbio Ltda",
        "INTEX BANK Banco de Câmbio S/A",
        "ISSUER Instituição de Pagamento Ltda",
        "ITAÚ UNIBANCO S/A",
        "IUGU Instituição de Pagamento S/A",
        "Intesa Sanpaolo Brasil S/A - Banco Múltiplo",
        "J17 - Sociedade de Crédito Direto S/A",
        "JM Corretora de Câmbio Ltda",
        "KIKAI Sociedade de Crédito Direto S/A",
        "KIWIFY Instituição de Pagamento Ltda",
        "KONECT Sociedade de Crédito Direto S/A",
        "Kirton Bank S/A - Banco Múltiplo",
        "LAMARA Sociedade de Crédito Direto S/A",
        "LAR Cooperativa de Crédito - LAR CREDI",
        "LB PAY Instituição de Pagamento Ltda",
        "LEND Sociedade de Crédito Direto S/A",
        "LEVYCAM - Corretora de Câmbio e Valores Ltda",
        "LIONS TRUST Corretora de Títulos e Valores Mobiliários Ltda",
        "LIQUIDO Instituição de Pagamento Ltda",
        "LISTO Sociedade de Crédito Direto S/A",
        "LOAN BRASIL Sociedade de Crédito Direto S/A",
        "Lastro RDV Corretora de Títulos e Valores Mobiliários Ltda",
        "MAF Distribuidora de Títulos e Valores Mobiliários S/A",
        "MAG Instituição de Pagamento Ltda",
        "MAGALUPAY Instituição de Pagamento S/A",
        "MAGNUM Sociedade de Crédito Direto S/A",
        "MAX Instituição de Pagamento Ltda",
        "MERCADO BITCOIN Instituição de Pagamento Ltda",
        "MERCADO PAGO Instituição de Pagamento Ltda",
        "MICROCASH Sociedade de Crédito ao Microempreendedor e à Empresa de Pequeno Porte Ltda",
        "MONERY Instituição de Pagamento S/A",
        "MONETARIE Sociedade de Crédito Direto S/A",
        "MONEYCORP Banco de Câmbio S/A",
        "MOVA Sociedade de Empréstimo entre Pessoas S/A",
        "MT Instituição de Pagamento SA",
        "MULTICRED Sociedade de Crédito Direto S/A",
        "MW Instituição de Pagamento Ltda",
        "MÉRITO Corretora de Títulos e Valores Mobiliários Ltda",
        "NEON Corretora de Títulos e Valores Mobiliários S/A",
        "NEON PagamentoS S/A - Instituição de Pagamento",
        "NIXFIN Sociedade de Crédito Direto S/A",
        "NU INVESTIMENTOS S/A - Corretora de Títulos e Valores Mobiliários",
        "NU PagamentoS S/A - Instituição de Pagamento",
        "NUMBRS Sociedade de Crédito Direto S/A",
        "NVIO BRASIL Sociedade de Crédito Direto S/A",
        "Nova Futura Corretora de Títulos e Valores Mobiliários Ltda",
        "Novo Banco Continental S/A - Banco Múltiplo",
        "OKTO Instituição de Pagamento S/A",
        "OLIVEIRA TRUST Distribuidora de Títulos e Valores Mobiliários S/A",
        "OM Corretora de Títulos e Valores Mobiliários Ltda",
        "ONEKEY PAYMENTS Instituição de Pagamento SA",
        "OPEA Sociedade de Crédito Direto S/A",
        "OSLO CAPITAL Distribuidora de Títulos e Valores Mobiliários S/A",
        "OURIBANK S/A Banco Múltiplo",
        "OXY Companhia Hipotecária",
        "OZ Corretora de Câmbio S/A",
        "Omni Banco S/A",
        "PAGARE Instituição de Pagamento S/A",
        "PAGME Instituição de Pagamento Ltda",
        "PAGPRIME Instituição de Pagamento Ltda",
        "PAGSEGURO INTERNET Instituição de Pagamento S/A",
        "PAGSMILE Instituição de Pagamento Ltda",
        "PAGUEVELOZ Instituição de Pagamento Ltda",
        "PARANÁ Banco S/A",
        "PAY Instituição de Pagamento S/A",
        "PAY4FUN Instituição de Pagamento S/A",
        "PEAK Sociedade de Empréstimo entre Pessoas S/A",
        "PERCAPITAL Sociedade de Crédito Direto S/A",
        "PICPAY BANK - Banco Múltiplo S/A",
        "PICPAY Instituição de Pagamento S/A",
        "PICPAY INVEST Corretora de Títulos e Valores Mobiliários Ltda",
        "PINBANK BRASIL Instituição de Pagamento S/A",
        "PLANNER Sociedade de Crédito Direto S/A",
        "PROTEGE CASH Instituição de Pagamento S/A",
        "PROTOTYPE Instituição de Pagamento S/A",
        "Planner Corretora de Valores S/A",
        "PÓLOCRED   Sociedade de Crédito ao Microempreendedor e à Empresa de Pequeno Porte Ltda",
        "QI Corretora de Títulos e Valores Mobiliários S/A",
        "QI Sociedade de Crédito Direto S/A",
        "QORE Distribuidora de Títulos e Valores Mobiliários Ltda",
        "QUADRA Sociedade de Crédito Direto S/A",
        "RB INVESTIMENTOS DISTRIBUIDORA DE TITULOS e Valores Mobiliários Ltda",
        "RED Sociedade de Crédito Direto S/A",
        "REPASSES FINANCEIROS e SOLUCOES TECNOLOGICAS Instituição de Pagamento S/A",
        "REVOLUT Sociedade de Crédito Direto S/A",
        "RJI Corretora de Títulos e Valores Mobiliários Ltda",
        "SAFETYPAY BRASIL Instituição de Pagamento Ltda",
        "SANTANDER Corretora de Títulos e Valores Mobiliários S/A",
        "SANTS Sociedade de Crédito Direto S/A",
        "SAYGO Corretora de Câmbio S/A",
        "SEFER INVESTIMENTOS Corretora de Títulos e Valores Mobiliários Ltda",
        "SENSO Corretora de Câmbio e Valores Mobiliários S/A",
        "SER FINANCE Sociedade de Crédito Direto S/A",
        "SETHI Sociedade de Crédito Direto S/A",
        "SIMPAUL Corretora de Câmbio e Valores Mobiliários  S/A",
        "SISPRIME DO BRASIL - Cooperativa de Crédito",
        "SMART SOLUTIONS GROUP Instituição de Pagamento Ltda",
        "SOCIAL BANK Banco Múltiplo S/A",
        "SOCRED S/A - Sociedade de Crédito ao Microempreendedor e à Empresa de Pequeno Porte",
        "SOMAPAY Sociedade de Crédito Direto S/A",
        "SQUID Sociedade de Crédito Direto S/A",
        "SRM BANK Instituição de Pagamento S/A",
        "STARK BANK S/A - Instituição de Pagamento",
        "STARK Sociedade de Crédito Direto S/A",
        "STATE STREET BRASIL S/A - Banco Comercial",
        "STONE Instituição de Pagamento S/A",
        "SUDACRED Sociedade de Crédito Direto S/A",
        "SUMUP Sociedade de Crédito Direto S/A",
        "SUPERLÓGICA Sociedade de Crédito Direto S/A",
        "SWAP Instituição de Pagamento S/A",
        "Scotiabank Brasil S/A Banco Múltiplo",
        "Secretaria do Tesouro Nacional - STN",
        "Sistema do Balcão B3",
        "TRANSFEERA Instituição de Pagamento S/A",
        "TRANSFERO Instituição de Pagamento Ltda",
        "TRINUS CAPITAL Distribuidora de Títulos e Valores Mobiliários S/A",
        "TRINUS Sociedade de Crédito Direto S/A",
        "TRIO Instituição de Pagamento Ltda",
        "TRUSTEE Corretora de Títulos e Valores Mobiliários Ltda",
        "TULLETT PREBON BRASIL Corretora de Valores e CÂMBIO Ltda",
        "TYCOON TECHNOLOGY Instituição de Pagamento S/A",
        "Terra Investimentos Corretora de Títulos e Valores Mobiliários Ltda",
        "UBS (BRASIL) Corretora de Valores S/A",
        "UBS BB Banco de Investimentos/A",
        "UNAVANTI Sociedade de Crédito Direto S/A",
        "UNIDA Corretora de Títulos e Valores Mobiliários Ltda",
        "UNIPRIME CENTRAL NACIONAL - CENTRAL NACIONAL DE Cooperativa de Crédito",
        "UP.P Sociedade de Empréstimo entre Pessoas S/A",
        "UY3 Sociedade de Crédito Direto S/A",
        "UZZIPAY Instituição de Pagamento S/A",
        "VERT Corretora de Títulos e Valores Mobiliários Ltda",
        "VIPS Corretora de Câmbio S/A",
        "VIS Corretora de Títulos e Valores Mobiliários Ltda",
        "VORTX Distribuidora de Títulos e Valores Mobiliários Ltda",
        "VUE Instituição de Pagamento S/A",
        "WARREN RENA Corretora de Títulos e Valores Mobiliários Ltda",
        "WASU - WALLET SUPPORT Instituição de Pagamento Ltda",
        "WE PAY OUT Instituição de Pagamento Ltda",
        "WNT CAPITAL Distribuidora de Títulos e Valores Mobiliários S/A",
        "WOOVI Instituição de Pagamento Ltda",
        "WX Instituição de Pagamento Ltda",
        "Z-ON Sociedade de Crédito Direto S/A",
        "Z1 Instituição de Pagamento Ltda",
        "ZERO Instituição de Pagamento S/A",
        "ZIPDIN Soluções DIGITAIS Sociedade de Crédito Direto S/A",
        "ÁGORA Corretora de Títulos e Valores Mobiliários S/A",
        "ÓTIMO Sociedade de Crédito Direto S/A",
    )



import re
import string

from typing import Pattern

from faker.providers.automotive import calculate_vin_str_weight
from faker.providers.automotive.de_AT import Provider as DeAtAutomotiveProvider
from faker.providers.automotive.de_DE import Provider as DeDeAutomotiveProvider
from faker.providers.automotive.es_ES import Provider as EsEsAutomotiveProvider
from faker.providers.automotive.es_MX import Provider as EsMxAutomotiveProvider
from faker.providers.automotive.mk_MK import Provider as MkMKAutomotiveProvider
from faker.providers.automotive.ro_RO import Provider as RoRoAutomotiveProvider
from faker.providers.automotive.ru_RU import Provider as RuRuAutomotiveProvider
from faker.providers.automotive.sk_SK import Provider as SkSkAutomotiveProvider
from faker.providers.automotive.tr_TR import Provider as TrTrAutomotiveProvider
from faker.providers.automotive.uk_UA import Provider as UkUaAutomotiveProvider


class _SimpleAutomotiveTestMixin:
    """Use this test mixin for simple license plate validation"""

    def perform_extra_checks(self, license_plate, match):
        pass

    def test_license_plate(self, faker, num_samples):
        for _ in range(num_samples):
            license_plate = faker.license_plate()
            match = self.license_plate_pattern.fullmatch(license_plate)
            assert match is not None
            self.perform_extra_checks(license_plate, match)

    def test_vin(self, faker, num_samples):
        for _ in range(num_samples):
            vin_number = faker.vin()
            # length check: 17
            assert len(vin_number) == 17

            # verify checksum: vin_number[8]
            front_part_weight = calculate_vin_str_weight(vin_number[:8], [8, 7, 6, 5, 4, 3, 2, 10])
            rear_part_weight = calculate_vin_str_weight(vin_number[9:], [9, 8, 7, 6, 5, 4, 3, 2])
            checksum = (front_part_weight + rear_part_weight) % 11
            checksum_str = "X" if checksum == 10 else str(checksum)
            assert vin_number[8] == checksum_str
            for char in vin_number[13:]:
                assert char in string.digits


class TestArBh(_SimpleAutomotiveTestMixin):
    """Test ar_BH automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"\d{6}")


class TestAzAz(_SimpleAutomotiveTestMixin):
    """Test az_AZ automotive provider methods"""

    license_plate_pattern = re.compile(r"\d{2}-[A-Z]{2}-\d{3}")


class TestArDz(_SimpleAutomotiveTestMixin):
    """Test ar_DZ automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"\d{5} [1-9]\d{2} (0[1-9]|[1-4]\d|5[0-8])")


class TestDeAt(_SimpleAutomotiveTestMixin):
    """Test de_AT automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"(?P<prefix>[A-Z]{1,2})-[1-9]{1}[0-9]{0,4} [A-Z]{1,3}")

    def perform_extra_checks(self, license_plate, match):
        assert match.group("prefix") in DeAtAutomotiveProvider.license_plate_prefix
        assert len(license_plate) in (8, 9)


class TestDeCh(_SimpleAutomotiveTestMixin):
    """Test de_CH automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2}-\d{1,3}\s?\d{0,3}")


class TestDeDe(_SimpleAutomotiveTestMixin):
    """Test de_DE automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"(?P<prefix>[A-Z\u00D6\u00DC]{1,3})-[A-Z]{1,2}-[1-9]{1,4}",
        re.UNICODE,
    )

    def perform_extra_checks(self, license_plate, match):
        assert match.group("prefix") in DeDeAutomotiveProvider.license_plate_prefix


class TestElGr(_SimpleAutomotiveTestMixin):
    """Test el_GR automotive provider methods"""

    license_plate_pattern = re.compile(r"^(?P<prefix>[A-Z]{2,3}) \d{4}$")


class TestEnPh(_SimpleAutomotiveTestMixin):
    """Test en_PH automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2}\d{4,5}|[A-Z]{3}\d{3,4}")
    motorcycle_pattern: Pattern = re.compile(r"[A-Z]{2}\d{4,5}")
    automobile_pattern: Pattern = re.compile(r"[A-Z]{3}\d{3,4}")

    def test_motorcycle_plate(self, faker, num_samples):
        for _ in range(num_samples):
            assert self.motorcycle_pattern.match(faker.motorcycle_license_plate())

    def test_automobile_plate(self, faker, num_samples):
        for _ in range(num_samples):
            assert self.automobile_pattern.match(faker.automobile_license_plate())

    def test_protocol_plate(self, faker, num_samples):
        for _ in range(num_samples):
            protocol_plate = faker.protocol_license_plate()
            assert int(protocol_plate) != 15 and 1 <= int(protocol_plate) <= 17


class TestEsCo(_SimpleAutomotiveTestMixin):
    """Test es_CO automotive provider methods"""

    license_plate_pattern = re.compile(r"[A-Z]{3}\d{3}|[A-Z]{3}\d{2}[A-Z]|T\d{4}|[A-Z]{2}\d{4}|R\d{5}|S\d{5}")


class TestEsEs:
    """Test es_ES automotive provider methods"""

    new_format_pattern: Pattern = re.compile(r"\d{4}\s[A-Z]{3}")
    old_format_pattern: Pattern = re.compile(r"(?P<province_prefix>[A-Z]{1,2})\s\d{4}\s[A-Z]{2}")

    def test_plate_new_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_unified()
            assert isinstance(plate, str)
            assert self.new_format_pattern.match(plate)

    def test_plate_old_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_by_province()
            assert isinstance(plate, str)
            match = self.old_format_pattern.match(plate)
            assert match
            assert match.group("province_prefix") in EsEsAutomotiveProvider.province_prefix

    def test_plate_old_format_explicit_province_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_by_province(province_prefix="CA")
            assert isinstance(plate, str)
            assert self.old_format_pattern.match(plate)
            assert plate[:2] == "CA"

    def test_plate_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate()
            assert isinstance(plate, str)
            assert self.new_format_pattern.match(plate) or self.old_format_pattern.match(plate)


class TestEsMx:
    """Test es_MX automotive provider methods"""

    # Letters allowed in Mexican serials (``I``, ``O`` and ``Q`` excluded).
    letters = EsMxAutomotiveProvider.license_plate_letters

    unified_pattern: Pattern = re.compile(rf"[{letters}]{{3}}-\d{{3}}-[{letters}]")
    old_pattern: Pattern = re.compile(rf"[{letters}]{{3}}-\d{{2}}-\d{{2}}")
    cdmx_pattern: Pattern = re.compile(rf"[{letters}]\d{{2}}-[{letters}]{{3}}")
    moto_national_pattern: Pattern = re.compile(rf"[{letters}]\d{{3}}[{letters}]")
    moto_cdmx_pattern: Pattern = re.compile(rf"\d{{4}} [{letters}]")
    taxi_lettered_pattern: Pattern = re.compile(rf"[{letters}]-\d{{3}}-[{letters}]{{3}}")
    taxi_numbered_pattern: Pattern = re.compile(rf"\d{{2}}-\d{{2}}-[{letters}]{{3}}")

    def test_plate_unified_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_unified()
            assert isinstance(plate, str)
            assert self.unified_pattern.fullmatch(plate)

    def test_plate_old_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_old()
            assert isinstance(plate, str)
            assert self.old_pattern.fullmatch(plate)

    def test_plate_cdmx_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_cdmx()
            assert isinstance(plate, str)
            assert self.cdmx_pattern.fullmatch(plate)

    def test_serial_letters_exclude_ioq(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate()
            for forbidden in ("I", "O", "Q"):
                # Only the letter positions matter; digits never contain these.
                assert forbidden not in re.sub(r"[\d-]", "", plate)

    def test_state_abbr_are_unique(self, faker):
        abbreviations = EsMxAutomotiveProvider.license_plate_state_abbrs
        # All 32 federal entities are represented exactly once.
        assert len(abbreviations) == 32
        assert len(set(abbreviations)) == 32
        assert tuple(EsMxAutomotiveProvider.license_plate_states.keys()) == abbreviations

    def test_state_names_are_complete(self, faker):
        states = EsMxAutomotiveProvider.license_plate_states
        assert len(states) == 32
        # Every code maps to a non-empty official name.
        for abbr, name in states.items():
            assert isinstance(abbr, str) and abbr
            assert isinstance(name, str) and name

    def test_license_plate_state_abbr_method(self, faker, num_samples):
        for _ in range(num_samples):
            abbr = faker.license_plate_state_abbr()
            assert abbr in EsMxAutomotiveProvider.license_plate_state_abbrs

    def test_license_plate_state_method(self, faker, num_samples):
        names = set(EsMxAutomotiveProvider.license_plate_states.values())
        for _ in range(num_samples):
            assert faker.license_plate_state() in names

    def test_plate_by_state_random(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_by_state()
            abbr, _, serial = plate.partition(" ")
            assert abbr in EsMxAutomotiveProvider.license_plate_state_abbrs
            assert self.unified_pattern.fullmatch(serial)

    def test_plate_by_state_explicit(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_by_state(state_abbr="JAL")
            assert plate.startswith("JAL ")
            assert self.unified_pattern.fullmatch(plate.split(" ", 1)[1])

    def test_motorcycle_plate_national_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.motorcycle_license_plate_national()
            assert isinstance(plate, str)
            assert self.moto_national_pattern.fullmatch(plate)

    def test_motorcycle_plate_cdmx_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.motorcycle_license_plate_cdmx()
            assert isinstance(plate, str)
            assert self.moto_cdmx_pattern.fullmatch(plate)

    def test_motorcycle_plate_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.motorcycle_license_plate()
            assert isinstance(plate, str)
            assert self.moto_national_pattern.fullmatch(plate) or self.moto_cdmx_pattern.fullmatch(plate)

    def test_public_transport_plate_lettered_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.public_transport_license_plate_lettered()
            assert isinstance(plate, str)
            assert self.taxi_lettered_pattern.fullmatch(plate)

    def test_public_transport_plate_numbered_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.public_transport_license_plate_numbered()
            assert isinstance(plate, str)
            assert self.taxi_numbered_pattern.fullmatch(plate)

    def test_public_transport_plate_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.public_transport_license_plate()
            assert isinstance(plate, str)
            assert self.taxi_lettered_pattern.fullmatch(plate) or self.taxi_numbered_pattern.fullmatch(plate)

    def test_plate_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate()
            assert isinstance(plate, str)
            assert (
                self.unified_pattern.fullmatch(plate)
                or self.old_pattern.fullmatch(plate)
                or self.cdmx_pattern.fullmatch(plate)
            )


class TestFiFi(_SimpleAutomotiveTestMixin):
    """Test fi_FI automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{3}-\d{3}")


class TestFilPh(TestEnPh):
    """Test fil_PH automotive provider methods"""

    pass


class TestFrDz(TestArDz):
    """Test fr_DZ automotive provider methods"""


class TestFrFr(_SimpleAutomotiveTestMixin):
    """Test fr_FR automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"\d{3}-[A-Z]{3}-\d{2}|[A-Z]{2}-\d{3}-[A-Z]{2}")


class TestHeIl(_SimpleAutomotiveTestMixin):
    license_plate_pattern: Pattern = re.compile(r"(\d{3}-\d{2}-\d{3})|(\d{2}-\d{3}-\d{2})")


class TestHuHu(_SimpleAutomotiveTestMixin):
    """Test hu_HU automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{3}-\d{3}")


class TestItIt(_SimpleAutomotiveTestMixin):
    """Test it_IT automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2}\d{3}[A-Z]{2}")


class TestJaJp(_SimpleAutomotiveTestMixin):
    """Test ja_JP automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"^(?:品川|足立|練馬|横浜|川崎|名古屋|大阪|神戸|福岡|札幌|尾張小牧|伊勢志摩) "
        r"\d{2,3} "
        r"(?:あ|い|う|え|か|き|く|け|こ|さ|す|せ|そ|た|ち|つ|て|と|な|に|ぬ|ね|の|は|ひ|ふ|ほ|"
        r"ま|み|む|め|も|や|ゆ|よ|ら|り|る|れ|ろ|わ|を) "
        r"(?:\d{2}-\d{2}|・{1,3}\d{1,3})$"
    )


class TestKoKr(_SimpleAutomotiveTestMixin):
    license_plate_pattern: Pattern = re.compile(
        r"^\d{2,3}[가나다라마거너더러머버서어저고노도로모보소오조구누두루무부수우주]\d{4}$"
    )


class TestNlBe(_SimpleAutomotiveTestMixin):
    """Test nl_BE automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"(\d{3}-[A-Z]{3})|" r"([A-Z]{3}-\d{3})|" r"([1-2]-[A-Z]{3}-\d{3})")


class TestNlNl(_SimpleAutomotiveTestMixin):
    """Test nl_NL automotive provider methods"""

    license_plate_car_pattern = re.compile(
        r"\d{2}-[BDFGHJKLNPRSTVXZ][A-Z]-[A-Z]{2}|"
        r"\d{2}-[BDFGHJKLNPRSTVXZ][A-Z]{2}-\d|"
        r"\d-[KSTVXZ][A-Z]{2}-\d{2}|"
        r"[BDFGHJKLNPRSTVXZ][A-Z]-\d{3}-[A-Z]|"
        r"[BDFGHJKLNPRSTVXZ]-\d{3}-[A-Z]{2}",
    )

    license_plate_motorbike_pattern = re.compile(
        r"M[A-Z]-[A-Z]{2}-\d{2}|" r"\d{2}-M[A-Z]-[A-Z]{2}",
    )

    license_plate_pattern = re.compile(
        license_plate_car_pattern.pattern + "|" + license_plate_motorbike_pattern.pattern,
    )

    def test_plate_car(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_car()
            assert isinstance(plate, str)
            assert self.license_plate_car_pattern.match(plate)

    def test_plate_motorbike(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_motorbike()
            assert isinstance(plate, str)
            assert self.license_plate_motorbike_pattern.match(plate)


class TestNoNo(_SimpleAutomotiveTestMixin):
    """Test no_NO automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2} \d{5}")


class TestPlPl:
    def test_License_plate(self, faker, num_samples):
        pattern: Pattern = re.compile(r"{patterns}".format(patterns="|".join(faker.license_plate_regex_formats())))
        for _ in range(num_samples):
            plate = faker.license_plate()
            assert pattern.fullmatch(plate)


class TestPtBr(_SimpleAutomotiveTestMixin):
    """Test pt_BR automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{3}-\d{1}[A-Z]{1}\d{2}")


class TestPtPt(_SimpleAutomotiveTestMixin):
    """Test pt_PT automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"\d{2}-\d{2}-[A-Z]{2}|" r"\d{2}-[A-Z]{2}-\d{2}|" r"[A-Z]{2}-\d{2}-\d{2}|" r"[A-Z]{2}-\d{2}-[A-Z]{2}",
    )


class TestRoRo(_SimpleAutomotiveTestMixin):
    """Test ro_RO automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"(?P<prefix>[A-Z]{1,2})-\d{2,3}-[A-Z]{3}")

    def perform_extra_checks(self, license_plate, match):
        assert match.group("prefix") in RoRoAutomotiveProvider.license_plate_prefix


class TestRuRu(_SimpleAutomotiveTestMixin):
    """Test ru_RU automotive provider methods"""

    _plate_letters = "".join(RuRuAutomotiveProvider.license_plate_letters)
    license_plate_pattern: Pattern = re.compile(
        r"(?:"
        r"(?P<private_plate_prefix>[{0}]\d\d\d[{0}][{0}])|"
        r"(?P<public_transport_plate_prefix>[{0}][{0}]\d\d\d)|"
        r"(?P<trailer_plate_prefix>[{0}][{0}]\d\d\d\d)|"
        r"(?P<police_plate_prefix>[{0}]\d\d\d\d)|"
        r"(?P<military_plate_prefix>\d\d\d\d[{0}][{0}])|"
        r"(?P<plate_number_special>00\dCD\d|00\dD\d\d\d|00\dT\d\d\d)"
        r") (?P<plate_suffix>.*)".format(_plate_letters),
    )

    def perform_extra_checks(self, license_plate, match):
        plate_suffix = match.group("plate_suffix")
        assert plate_suffix in RuRuAutomotiveProvider.license_plate_suffix

    def test_vehicle_category(self, faker, num_samples):
        for _ in range(num_samples):
            vehicle_category = faker.vehicle_category()
            assert isinstance(vehicle_category, str)
            assert vehicle_category in RuRuAutomotiveProvider.vehicle_categories


class TestSkSk(_SimpleAutomotiveTestMixin):
    """Test sk_SK automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"(?P<prefix>[A-Z]{2})\d{3}[A-Z]{2}")

    def perform_extra_checks(self, license_plate, match):
        assert match.group("prefix") in SkSkAutomotiveProvider.license_plate_prefix


class TestSqAl(_SimpleAutomotiveTestMixin):
    """Test sq_AL automotive providers methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2} \d{3}[A-Z]{2}")


class TestSvSe(_SimpleAutomotiveTestMixin):
    """Test sv_SE automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{3} \d{2}[\dA-Z]")


class TestThTh(_SimpleAutomotiveTestMixin):
    """Test th_TH automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"(\d [ก-ฮ]{2} \d{1,4})|"  # car
        r"([ก-ฮ]{2} \d{1,4})|"  # car
        r"([ก-ฮ]{3} \d{1,3})|"  # motorcycle
        r"(\d{2}-\d{4})",  # truck
    )


class TestTlPh(TestEnPh):
    """Test tl_PH automotive provider methods"""

    pass


class TestTrTr(_SimpleAutomotiveTestMixin):
    """Test tr_TR automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"\d{2} [A-Z] \d{4}|"
        r"\d{2} [A-Z] \d{5}|"
        r"\d{2} [A-Z]{2} \d{3}|"
        r"\d{2} [A-Z]{2} \d{4}|"
        r"\d{2} [A-Z]{3} \d{2}|"
        r"\d{2} [A-Z]{3} \d{3}",
    )

    def perform_extra_checks(self, license_plate, match):
        [city_code, letters, _] = license_plate.split(" ")
        assert int(city_code) in range(1, 82)
        assert all(letter in TrTrAutomotiveProvider.ascii_uppercase_turkish for letter in letters)


class TestUkUa(_SimpleAutomotiveTestMixin):
    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2}\d{4}[A-Z]{2}")

    def perform_extra_checks(self, license_plate, match):
        assert license_plate[-2:] in UkUaAutomotiveProvider.license_plate_suffix

    def test_temporary_plate(self, faker, num_samples):
        pattern = r"\d{2} [A-Z]{2}\d{4}"

        for _ in range(num_samples):
            temporary = faker.license_plate(temporary_plate=True)
            match = re.search(pattern, temporary)
            assert match is not None

    def test_diplomatic_plate(self, faker, num_samples):
        pattern = r"(CDP \d{3})|(DP|S) \d{3} \d{3}"

        for _ in range(num_samples):
            temporary = faker.diplomatic_license_plate()
            match = re.search(pattern, temporary)
            assert match is not None

    def test_prefix(self, faker):
        for _ in range(10):
            temporary = faker.plate_letter_prefix(region_name="Lviv")
            assert len(temporary) == 2
            assert temporary in UkUaAutomotiveProvider.license_region_data.get("Lviv")[0]

    def test_region_code(self, faker):
        assert "14" == faker.plate_region_code(region_name="Lviv")


class TestViVn(_SimpleAutomotiveTestMixin):
    """Test vi_VN automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"\d{2}[ABCDĐEFGHKLMNPSTUVXYZ]-\d{5}")


class TestZhCn(_SimpleAutomotiveTestMixin):
    """Test zh_CN automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"^[京津冀晋蒙辽吉黑沪苏浙皖闽赣鲁豫鄂湘粤桂琼川贵云渝藏陕甘青宁新]{1}[A-Z]{1}-[A-Z0-9]{5}"
    )


class TestZhTw(_SimpleAutomotiveTestMixin):
    """Test zh_TW automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"([A-Z]{2}-\d{4})|"  # prior 2012 v1
        r"(\d{4}-[A-Z]{2})|"  # prior 2012 v2
        r"([A-Z]{3}-\d{4})|"  # new format since 2014
        r"([A-Z]{3}-\d{3})",  # commercial cars since 2012
    )


class TestMkMk(_SimpleAutomotiveTestMixin):
    """Test mk_MK automotive provider methods"""

    # Format: XX NNNN YY  (2-letter region, 4 digits, 2 letters)
    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2} \d{4} [A-BCDEFGHIJKLMNOPRSTUVZ]{2}")

    def test_license_plate_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate()
            prefix = plate.split()[0]
            assert prefix in MkMKAutomotiveProvider.license_plate_prefixes
